import unittest
import sys
if sys.version_info[0] == 2:
    from itertools import izip as zip
import rpy2.rlike.container as rlc

class OrdDictTestCase(unittest.TestCase):

    def testNew(self):
        nl = rlc.OrdDict()

        x = (('a', 123), ('b', 456), ('c', 789))
        nl = rlc.OrdDict(x)

    def testLen(self):
        x = rlc.OrdDict()
        self.assertEqual(0, len(x))

        x['a'] = 2
        x['b'] = 1

        self.assertEqual(2, len(x))

    def testGetSetitem(self):
        x = rlc.OrdDict()
        
        x['a'] = 1
        self.assertEqual(1, len(x))
        self.assertEqual(1, x['a'])
        self.assertEqual(0, x.index('a'))
        x['a'] = 2
        self.assertEqual(1, len(x))
        self.assertEqual(2, x['a'])
        self.assertEqual(0, x.index('a'))
        x['b'] = 1
        self.assertEqual(2, len(x))
        self.assertEqual(1, x['b'])
        self.assertEqual(1, x.index('b'))

    def testGet(self):
        x = rlc.OrdDict()
        x['a'] = 1
        self.assertEqual(1, x.get('a'))
        self.assertEqual(None, x.get('b'))
        self.assertEqual(2, x.get('b', 2))
        
    def testKeys(self):
        x = rlc.OrdDict()
        for i,k in enumerate('abcdef'):
            x[k] = i
        for i,k in enumerate(x.keys()):
            self.assertEqual('abcdef'[i], k)

    def testGetSetitemWithNone(self):
        x = rlc.OrdDict()
        
        x['a'] = 1
        x[None] = 2
        self.assertEqual(2, len(x))
        x['b'] = 5
        self.assertEqual(3, len(x))
        self.assertEqual(1, x['a'])
        self.assertEqual(5, x['b'])
        self.assertEqual(0, x.index('a'))
        self.assertEqual(2, x.index('b'))
        
    def testReverse(self):
        x = rlc.OrdDict()
        x['a'] = 3
        x['b'] = 2
        x['c'] = 1
        x.reverse()
        self.assertEqual(1, x['c'])
        self.assertEqual(0, x.index('c'))
        self.assertEqual(2, x['b'])
        self.assertEqual(1, x.index('b'))
        self.assertEqual(3, x['a'])
        self.assertEqual(2, x.index('a'))

    def testItems(self):
        args = (('a', 5), ('b', 4), ('c', 3),
                ('d', 2), ('e', 1))
        x = rlc.OrdDict(args)
        it = x.items()
        for ki, ko in zip(args, it):
            self.assertEqual(ki[0], ko[0])
            self.assertEqual(ki[1], ko[1])

class TaggedListTestCase(unittest.TestCase):

    def test__add__(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        tl = tl + tl
        self.assertEqual(6, len(tl))
        self.assertEqual(('a', 'b', 'c', 'a', 'b', 'c'), tl.tags)
        self.assertEqual((1,2,3,1,2,3), tuple(tl))

    def test__delitem__(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        self.assertEqual(3, len(tl))
        del tl[1]
        self.assertEqual(2, len(tl))
        self.assertEqual(tl.tags, ('a', 'c'))
        self.assertEqual(tuple(tl), (1, 3))

    def test__delslice__(self):
        tl = rlc.TaggedList((1,2,3,4), tags=('a', 'b', 'c', 'd'))        
        del tl[1:3]
        self.assertEqual(2, len(tl))
        self.assertEqual(tl.tags, ('a', 'd'))
        self.assertEqual(tuple(tl), (1, 4))

    def test__iadd__(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        tl += tl
        self.assertEqual(6, len(tl))
        self.assertEqual(('a', 'b', 'c', 'a', 'b', 'c'), tl.tags)
        self.assertEqual((1,2,3,1,2,3), tuple(tl))

    def test__imul__(self):
        tl = rlc.TaggedList((1,2), tags=('a', 'b'))
        tl *= 3
        self.assertEqual(6, len(tl))
        self.assertEqual(('a', 'b', 'a', 'b', 'a', 'b'), tl.tags)
        self.assertEqual((1,2,1,2,1,2), tuple(tl))

    def test__init__(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        self.assertRaises(ValueError, rlc.TaggedList, (1,2,3), tags = ('b', 'c'))

    def test__setslice__(self):
        tl = rlc.TaggedList((1,2,3,4), tags=('a', 'b', 'c', 'd'))        
        tl[1:3] = [5, 6]
        self.assertEqual(4, len(tl))
        self.assertEqual(tl.tags, ('a', 'b', 'c', 'd'))
        self.assertEqual(tuple(tl), (1, 5, 6, 4))

    def testappend(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        self.assertEqual(3, len(tl))
        tl.append(4, tag='a')
        self.assertEqual(4, len(tl))
        self.assertEqual(4, tl[3])
        self.assertEqual(('a', 'b', 'c', 'a'), tl.tags)

    def testextend(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        tl.extend([4, 5])
        self.assertEqual(('a', 'b', 'c', None, None), tuple(tl.itertags()))
        self.assertEqual((1, 2, 3, 4, 5), tuple(tl))

    def testinsert(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        tl.insert(1, 4, tag = 'd')
        self.assertEqual(('a', 'd', 'b', 'c'), tuple(tl.itertags()))
        self.assertEqual((1, 4, 2, 3), tuple(tl))
        
    def testitems(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))        
        self.assertEqual((('a', 1), ('b', 2), ('c', 3)), 
                          tuple(tl.items()))

    def testiterontag(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'a'))
        self.assertEqual((1, 3), tuple(tl.iterontag('a')))

    def testitertags(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))
        self.assertEqual(('a', 'b', 'c'), tuple(tl.itertags()))

    def testpop(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))
        self.assertEqual(3, len(tl))
        elt = tl.pop()
        self.assertEqual(3, elt)
        self.assertEqual(2, len(tl))
        self.assertEqual(tl.tags, ('a', 'b'))
        self.assertEqual(tuple(tl), (1, 2))

        elt = tl.pop(0)
        self.assertEqual(1, elt)
        self.assertEqual(1, len(tl))
        self.assertEqual(tl.tags, ('b', ))

    def testremove(self):
        tl = rlc.TaggedList((1,2,3), tags=('a', 'b', 'c'))
        self.assertEqual(3, len(tl))
        tl.remove(2)
        self.assertEqual(2, len(tl))
        self.assertEqual(tl.tags, ('a', 'c'))
        self.assertEqual(tuple(tl), (1, 3))

    def testreverse(self):
        tn = ['a', 'b', 'c']
        tv = [1,2,3]
        tl = rlc.TaggedList(tv, tags = tn)
        tl.reverse()
        self.assertEqual(3, len(tl))
        self.assertEqual(tl.tags, ('c', 'b', 'a'))
        self.assertEqual(tuple(tl), (3, 2, 1))

    def testsort(self):
        tn = ['a', 'c', 'b']
        tv = [1,3,2]
        tl = rlc.TaggedList(tv, tags = tn)
        tl.sort()

        self.assertEqual(tl.tags, ('a', 'b', 'c'))
        self.assertEqual(tuple(tl), (1, 2, 3))
        
    def testtags(self):
        tn = ['a', 'b', 'c']
        tv = [1,2,3]
        tl = rlc.TaggedList(tv, tags = tn)
        tags = tl.tags
        self.assertTrue(isinstance(tags, tuple))
        self.assertEqual(tags, ('a', 'b', 'c'))

        tn = ['d', 'e', 'f']
        tl.tags = tn
        self.assertTrue(isinstance(tags, tuple))
        self.assertEqual(tuple(tn), tl.tags)

    def testsettag(self):
        tn = ['a', 'b', 'c']
        tv = [1,2,3]
        tl = rlc.TaggedList(tv, tags = tn)
        tl.settag(1, 'z')
        self.assertEqual(tl.tags, ('a', 'z', 'c'))

    def testfrom_items(self):
        od = rlc.OrdDict( (('a', 1), ('b', 2), ('c', 3)) )
        tl = rlc.TaggedList.from_items(od)
        self.assertEqual(('a', 'b', 'c'), tl.tags)
        self.assertEqual((1, 2, 3), tuple(tl))

        tl = rlc.TaggedList.from_items({'a':1, 'b':2, 'c':3})
        self.assertEqual(set(('a', 'b', 'c')), set(tl.tags))
        self.assertEqual(set((1, 2, 3)), set(tuple(tl)))
        
def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(OrdDictTestCase)
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(TaggedListTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()



import unittest

# Try to load R dplyr package, and see if it works
from rpy2.rinterface import RRuntimeError
has_dplyr = None
try:
    from rpy2.robjects.lib import dplyr
    has_dplyr = True
except RRuntimeError:
    has_dplyr = False

from rpy2.robjects.packages import importr, data
datasets = importr('datasets')
mtcars = data(datasets).fetch('mtcars')['mtcars']

@unittest.skipUnless(has_dplyr, 'dplyr package not available in R')
class DplyrTestCase(unittest.TestCase):

    def testSetup(self):
        pass

    def tearDown(self):
        pass

    def testDataFrame(self):
        dataf = dplyr.DataFrame(mtcars)
        # FIXME: no testing much at the moment...
        self.assertTrue(isinstance(dataf, dplyr.DataFrame))

    def testFilter_NoFilter(self):
        dataf = dplyr.DataFrame(mtcars)
        dataf_filter = dataf.filter()
        self.assertEqual(dataf.nrow, dataf_filter.nrow)

    def testFilter_OneFilter(self):
        dataf = dplyr.DataFrame(mtcars)
        ngear_gt_3 = len(tuple(x for x in dataf.rx2('gear') if x > 3))
        dataf_filter = dataf.filter('gear > 3')        
        self.assertEqual(ngear_gt_3, dataf_filter.nrow)

    def testSplitMergeFunction(self):
        dataf = dplyr.DataFrame(mtcars)
        dataf_by_gear = dataf.group_by('gear')
        dataf_sum_gear = dataf_by_gear.summarize(foo='sum(gear)')
        self.assertEquals(type(dataf_sum_gear), dplyr.DataFrame)
    
    def testJoin(self):
        dataf_a = dplyr.DataFrame(mtcars)
        dataf_b = dataf_a.mutate(foo=1)
        dataf_c = dataf_a.inner_join(dataf_b)
        all_names = list(dataf_a.colnames)
        all_names.append('foo')
        try:
            # Python 3
            self.assertCountEqual(all_names, dataf_c.colnames)
        except AttributeError as ae:
            # Python 2.7
            self.assertItemsEqual(all_names, dataf_c.colnames)

    def testCollect(self):
        dataf = dplyr.DataFrame(mtcars)
        dataf_collected = dataf.collect()
        # FIXME: no real test here. Just ensuring that it is returning
        #        without error
        self.assertEquals(dplyr.DataFrame, type(dataf_collected))
        
    def testCollect(self):
        dataf = dplyr.DataFrame(mtcars)
        dataf_collected = dataf.collect()
        # FIXME: no real test here. Just ensuring that it is returning
        #        without error
        self.assertEquals(dplyr.DataFrame, type(dataf_collected))

        
def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(DplyrTestCase)
    return suite

if __name__ == '__main__':
     unittest.main()



import unittest
import rpy2.robjects as robjects
rinterface = robjects.rinterface
import array

identical = rinterface.baseenv["identical"]
Function = robjects.functions.Function
SignatureTranslatedFunction = robjects.functions.SignatureTranslatedFunction

class FunctionTestCase(unittest.TestCase):
    def testNewInvalid(self):
        self.assertRaises(ValueError, Function, 'a')

    def testNewInvalid(self):
        ri_f = rinterface.baseenv.get('help')
        
        ro_f = Function(ri_f)
        
        self.assertTrue(identical(ri_f, ro_f))

    def testCall(self):
        ri_f = rinterface.baseenv.get('sum')
        ro_f = Function(ri_f)
        
        ro_v = robjects.Vector(array.array('i', [1,2,3]))
        
        s = ro_f(ro_v)

    def testCallWithSexp(self):
        ro_f = robjects.baseenv["sum"]
        ri_vec = robjects.rinterface.SexpVector([1,2,3], 
                                                robjects.rinterface.INTSXP)
        res = ro_f(ri_vec)
        self.assertEqual(6, res[0])

    def testCallClosureWithRObject(self):
        ri_f = rinterface.baseenv["sum"]
        ro_vec = robjects.Vector(array.array('i', [1,2,3]))
        res = ri_f(ro_vec)
        self.assertEqual(6, res[0])

    def testFormals(self):
        ri_f = robjects.r('function(x, y) TRUE')
        res = ri_f.formals()
        #FIXME: no need for as.list when paired list are handled
        res = robjects.r['as.list'](res)
        self.assertEqual(2, len(res))
        n = res.names
        self.assertEqual("x", n[0])
        self.assertEqual("y", n[1])

class SignatureTranslatedFunctionTestCase(unittest.TestCase):
    def testNewInvalid(self):
        self.assertRaises(ValueError, 
                          SignatureTranslatedFunction, 'a')

    def testNew(self):
        ri_f = rinterface.baseenv.get('rank')
        ro_f = SignatureTranslatedFunction(ri_f)        
        self.assertTrue(identical(ri_f, ro_f))

    @unittest.expectedFailure
    def testNewWithTranslation(self):
        ri_f = rinterface.baseenv.get('rank')
        ro_f = SignatureTranslatedFunction(ri_f,
                                           prm_translate = {'foo_bar': 'na.last'})
        self.assertTrue(identical(ri_f, ro_f))

    def testCall(self):
        ri_f = rinterface.baseenv.get('sum')
        ro_f = robjects.Function(ri_f)
        
        ro_v = robjects.Vector(array.array('i', [1,2,3]))
        
        s = ro_f(ro_v)

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(FunctionTestCase)
    return suite

if __name__ == '__main__':
     unittest.main()



import unittest
import copy
import gc
from rpy2 import rinterface

rinterface.initr()

class SexpTestCase(unittest.TestCase):

    def testNew_invalid(self):

        x = "a"
        self.assertRaises(ValueError, rinterface.Sexp, x)

    def testNew(self):
        sexp = rinterface.baseenv.get("letters")
        sexp_new = rinterface.Sexp(sexp)

        idem = rinterface.baseenv.get("identical")
        self.assertTrue(idem(sexp, sexp_new)[0])

        sexp_new2 = rinterface.Sexp(sexp)
        self.assertTrue(idem(sexp, sexp_new2)[0])
        del(sexp)
        self.assertTrue(idem(sexp_new, sexp_new2)[0])


    def testTypeof_get(self):
        sexp = rinterface.baseenv.get("letters")
        self.assertEqual(sexp.typeof, rinterface.STRSXP)
        
        sexp = rinterface.baseenv.get("pi")
        self.assertEqual(sexp.typeof, rinterface.REALSXP)
        
        sexp = rinterface.baseenv.get("plot")
        self.assertEqual(sexp.typeof, rinterface.CLOSXP)

    def testList_attrs(self):
        x = rinterface.IntSexpVector((1,2,3))
        self.assertEqual(0, len(x.list_attrs()))
        x.do_slot_assign('a', rinterface.IntSexpVector((33,)))
        self.assertEqual(1, len(x.list_attrs()))
        self.assertTrue('a' in x.list_attrs())

    def testDo_slot(self):
        data_func = rinterface.baseenv.get("data")
        data_func(rinterface.SexpVector(["iris", ], rinterface.STRSXP))
        sexp = rinterface.globalenv.get("iris")
        names = sexp.do_slot("names")
        iris_names = ("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width", "Species")

        self.assertEqual(len(iris_names), len(names))

        for i, n in enumerate(iris_names):
            self.assertEqual(iris_names[i], names[i])

        self.assertRaises(LookupError, sexp.do_slot, "foo")

    def testDo_slot_emptyString(self):
        sexp = rinterface.baseenv.get('pi')
        self.assertRaises(ValueError, sexp.do_slot, "")

    def testDo_slot_assign(self):
        data_func = rinterface.baseenv.get("data")
        data_func(rinterface.SexpVector(["iris", ], rinterface.STRSXP))
        sexp = rinterface.globalenv.get("iris")
        iris_names = rinterface.StrSexpVector(['a', 'b', 'c', 'd', 'e'])
        sexp.do_slot_assign("names", iris_names)
        names = [x for x in sexp.do_slot("names")]
        self.assertEqual(['a', 'b', 'c', 'd', 'e'], names)

    def testDo_slot_assign_create(self):
        #test that assigning slots is also creating the slot
        x = rinterface.IntSexpVector([1,2,3])
        x.do_slot_assign("foo", rinterface.StrSexpVector(["bar", ]))
        slot = x.do_slot("foo")
        self.assertEqual(1, len(slot))
        self.assertEqual("bar", slot[0])

    def testDo_slot_assign_emptyString(self):
        #test that assigning slots is also creating the slot
        x = rinterface.IntSexpVector([1,2,3])
        self.assertRaises(ValueError, 
                          x.do_slot_assign, "", 
                          rinterface.StrSexpVector(["bar", ]))

    def testSexp_rsame_true(self):
        sexp_a = rinterface.baseenv.get("letters")
        sexp_b = rinterface.baseenv.get("letters")
        self.assertTrue(sexp_a.rsame(sexp_b))

    def testSexp_rsame_false(self):
        sexp_a = rinterface.baseenv.get("letters")
        sexp_b = rinterface.baseenv.get("pi")
        self.assertFalse(sexp_a.rsame(sexp_b))

    def testSexp_rsame_wrongType(self):
        sexp_a = rinterface.baseenv.get("letters")
        self.assertRaises(ValueError, sexp_a.rsame, 'foo')

    def testSexp_sexp(self):
        sexp = rinterface.IntSexpVector([1,2,3])
        sexp_count = sexp.__sexp_refcount__
        sexp_cobj = sexp.__sexp__
        d = dict(rinterface._rinterface.protected_rids())
        self.assertEqual(sexp_count, d[sexp.rid])
        self.assertEqual(sexp_count, sexp.__sexp_refcount__)
        sexp2 = rinterface.IntSexpVector([4,5,6,7])
        sexp2_rid = sexp2.rid
        sexp2.__sexp__ = sexp_cobj
        del(sexp)
        gc.collect()
        d = dict(rinterface._rinterface.protected_rids())
        self.assertEqual(None, d.get(sexp2_rid))

    def testSexp_rclass_get(self):
        sexp = rinterface.baseenv.get("letters")
        self.assertEqual(len(sexp.rclass), 1)
        self.assertEqual(sexp.rclass[0], "character")
        sexp = rinterface.baseenv.get("matrix")(0)
        self.assertEqual(len(sexp.rclass), 1)
        self.assertEqual(sexp.rclass[0], "matrix")

    def testSexp_rclass_set(self):
        sexp = rinterface.IntSexpVector([1,2,3])
        sexp.rclass = rinterface.StrSexpVector(['foo'])
        self.assertEqual(len(sexp.rclass), 1)
        self.assertEqual(sexp.rclass[0], "foo")
        
    def testSexp_sexp_wrongtypeof(self):
        sexp = rinterface.IntSexpVector([1,2,3])
        cobj = sexp.__sexp__
        sexp = rinterface.StrSexpVector(['a', 'b'])
        self.assertEqual(2, len(sexp))
        self.assertRaises(ValueError, sexp.__setattr__, '__sexp__', cobj)


    def testSexp_sexp_UniqueCapsule(self):
        sexp = rinterface.IntSexpVector([1,2,3])
        sexp_count = sexp.__sexp_refcount__
        cobj = sexp.__sexp__
        # check that no increase in the refcount: the capsule is unique
        self.assertEqual(sexp_count, sexp.__sexp_refcount__)
        self.assertEqual(sexp_count, 
                         dict(rinterface.protected_rids())[sexp.rid])
        del(cobj)
        gc.collect()
        self.assertEqual(sexp_count, sexp.__sexp_refcount__)
        self.assertEqual(sexp_count, 
                         dict(rinterface.protected_rids())[sexp.rid])
        sexp_rid = sexp.rid
        del(sexp)
        gc.collect()
        self.assertFalse(sexp_rid in dict(rinterface.protected_rids()))
        

    def testSexp_sexp_set(self):
        x = rinterface.IntSexpVector([1,2,3])
        x_s = x.__sexp__
        x_rid = x.rid
        # The Python reference count of the capsule is incremented,
        # not the rpy2 reference count
        self.assertEqual(1, x.__sexp_refcount__)
        y = rinterface.IntSexpVector([4,5,6])
        y_count = y.__sexp_refcount__
        y_rid = y.rid
        self.assertEqual(1, y_count)
        self.assertTrue(x_rid in [elt[0] for elt in rinterface.protected_rids()])
        x.__sexp__ = y.__sexp__
        self.assertFalse(x_rid in [elt[0] for elt in rinterface.protected_rids()])
        self.assertEqual(x.rid, y.rid)
        self.assertEqual(y_rid, y.rid)
        # now both x and y point to the same capsule, making
        # the rpy2 reference count to 2
        self.assertEqual(x.__sexp_refcount__, y.__sexp_refcount__)
        self.assertEqual(y_count+1, x.__sexp_refcount__)
        del(x)
        self.assertTrue(y_rid in [elt[0] for elt in rinterface.protected_rids()])
        del(y)
        self.assertFalse(y_rid in [elt[0] for elt in rinterface.protected_rids()])
        
    def testSexp_deepcopy(self):
        sexp = rinterface.IntSexpVector([1,2,3])
        self.assertEqual(0, sexp.named)
        rinterface.baseenv.get("identity")(sexp)
        self.assertEqual(2, sexp.named)
        sexp2 = sexp.__deepcopy__()
        self.assertEqual(sexp.typeof, sexp2.typeof)
        self.assertEqual(list(sexp), list(sexp2))
        self.assertFalse(sexp.rsame(sexp2))
        self.assertEqual(0, sexp2.named)
        # should be the same as above, but just in case:
        sexp3 = copy.deepcopy(sexp)
        self.assertEqual(sexp.typeof, sexp3.typeof)
        self.assertEqual(list(sexp), list(sexp3))
        self.assertFalse(sexp.rsame(sexp3))
        self.assertEqual(0, sexp3.named)

    def testRID(self):
        globalenv_id = rinterface.baseenv.get('.GlobalEnv').rid
        self.assertEqual(globalenv_id, rinterface.globalenv.rid)
        
class RNULLTestCase(unittest.TestCase):
    def testRNULLType_nonzero(self):
        NULL = rinterface.RNULLType()
        self.assertFalse(NULL)

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(SexpTestCase)
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(RNULLTestCase))
    return suite

if __name__ == '__main__':
    tr = unittest.TextTestRunner(verbosity = 2)
    tr.run(suite())



import unittest
import sys
if sys.version_info[0] == 2:
    from itertools import izip as zip
import rpy2.rlike.indexing as rfi

class OrderTestCase(unittest.TestCase):

    def testOrder(self):
        seq  = (  2,   1,   5,   3,   4)
        expected = (1, 2, 3, 4, 5)
        res = rfi.order(seq)
        for va, vb in zip(expected, res):
            self.assertEqual(va, seq[vb])


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(OrderTestCase)
    #suite.addTest(unittest.TestLoader().loadTestsFromTestCase(VectorizeTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()



from collections import namedtuple
from six import with_metaclass
from rpy2.robjects.packages import (importr, data,
                                    Package, default_symbol_r2python,
                                    default_symbol_check_after,
                                    WeakPackage)
import warnings
    
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    dplyr = importr('dplyr', on_conflict="warn")
    lazyeval = importr('lazyeval', on_conflict="warn")
    dplyr = WeakPackage(dplyr._env,
                        dplyr.__rname__,
                        translation=dplyr._translation,
                        exported_names=dplyr._exported_names,
                        on_conflict="warn",
                        version=dplyr.__version__,
                        symbol_r2python=dplyr._symbol_r2python,
                        symbol_check_after=dplyr._symbol_check_after)
TARGET_VERSION = '0.5.0'
if dplyr.__version__ != TARGET_VERSION:
    warnings.warn('This was designed againt dplyr version %s but you have %s' % (TARGET_VERSION, dplyr.__version__))

    
from rpy2 import robjects

StringInEnv = namedtuple('StringInEnv', 'string env')

def _fix_args_inenv(args, env):
    args_inenv = list()
    for v in args:
        if isinstance(v, StringInEnv):
            args_inenv.append(lazyeval.as_lazy(v.string, env=v.env))
        else:
            args_inenv.append(lazyeval.as_lazy(v, env=env))
    return args_inenv

def _fix_kwargs_inenv(kwargs, env):
    kwargs_inenv = dict()
    for k,v in kwargs.items():
        if isinstance(v, StringInEnv):
            kwargs_inenv[k] = lazyeval.as_lazy(v.string, env=v.env)
        else:
            kwargs_inenv[k] = lazyeval.as_lazy(v, env=env)
    return kwargs_inenv

def _wrap(rfunc, cls, env=robjects.globalenv):
    def func(dataf, *args, **kwargs):
        args_inenv = _fix_args_inenv(args, env)
        kwargs_inenv = _fix_kwargs_inenv(kwargs, env)
        if cls is None:
            return type(dataf)(rfunc(dataf, *args_inenv, **kwargs_inenv))
        else:
            return cls(rfunc(dataf, *args_inenv, **kwargs_inenv))
    return func

def _wrap2(rfunc, cls, env=robjects.globalenv):
    def func(dataf_a, dataf_b, *args, **kwargs):
        args_inenv = _fix_args_inenv(args, env)
        kwargs_inenv = _fix_kwargs_inenv(kwargs, env)
        if cls is None:
            return type(dataf_a)(rfunc(dataf_a, dataf_b,
                                       *args_inenv, **kwargs_inenv))
        else:
            return cls(rfunc(dataf_a, dataf_b,
                             *args_inenv, **kwargs_inenv))
    return func

def _make_pipe(rfunc, cls, env=robjects.globalenv):
    def func(*args, **kwargs):
        def func2(obj):
            args_inenv = _fix_args_inenv(args, env)
            kwargs_inenv = _fix_kwargs_inenv(kwargs, env)
            return cls(rfunc(obj, *args_inenv, **kwargs_inenv))
        return func2
    return func

def _make_pipe2(rfunc, cls, env=robjects.globalenv):
    def func(*args, **kwargs):
        def func2(obj_a, obj_b):
            args_inenv = _fix_args_inenv(args, env)
            kwargs_inenv = _fix_kwargs_inenv(kwargs, env)
            return cls(rfunc(obj_a, obj_b, *args_inenv, **kwargs_inenv))
        return func2
    return func

class DataFrame(robjects.DataFrame):
    """ DataFrame object extending the object of the same name in
    `rpy2.robjects.vectors` with functionalities defined in the R
    package `dplyr`."""
    
    def __rshift__(self, other):
        return other(self)

    def copy_to(self, destination, name, **kwargs):
        """
        - destination: database
        - name: table name in the destination database
        """
        res = dplyr.copy_to(destination, self, name=name)
        return type(self)(res)

    def collapse(self, *args, **kwargs):
        """
        Call the function `collapse` in the R package `dplyr`.
        """
        
        cls = type(self)
        return cls(dplyr.collapse(self, *args, **kwargs))

    def collect(self, *args, **kwargs):
        """Call the function `collect` in the R package `dplyr`."""
        cls = type(self)
        return cls(dplyr.collect(self, *args, **kwargs))
        
class GroupedDataFrame(DataFrame):
    pass

DataFrame.arrange = _wrap(dplyr.arrange_, None)
DataFrame.distinct = _wrap(dplyr.distinct_, None)
DataFrame.mutate = _wrap(dplyr.mutate_, None)
DataFrame.transmute = _wrap(dplyr.transmute_, None)
DataFrame.filter = _wrap(dplyr.filter_, None)
DataFrame.select = _wrap(dplyr.select_, None)
DataFrame.group_by = _wrap(dplyr.group_by_, GroupedDataFrame)
DataFrame.distinct = _wrap(dplyr.distinct_, None)
DataFrame.inner_join = _wrap2(dplyr.inner_join, None)
DataFrame.left_join = _wrap2(dplyr.left_join, None)
DataFrame.right_join = _wrap2(dplyr.right_join, None)
DataFrame.full_join = _wrap2(dplyr.full_join, None)
DataFrame.semi_join = _wrap2(dplyr.semi_join, None)
DataFrame.anti_join = _wrap2(dplyr.anti_join, None)
DataFrame.union = _wrap2(dplyr.union, None)
DataFrame.intersect = _wrap2(dplyr.intersect, None)
DataFrame.setdiff = _wrap2(dplyr.setdiff, None)

DataFrame.sample_n = _wrap(dplyr.sample_n, None)
DataFrame.sample_frac = _wrap(dplyr.sample_frac, None)
DataFrame.slice = _wrap(dplyr.slice_, None)

DataFrame.count = _wrap(dplyr.count_, None)
DataFrame.tally = _wrap(dplyr.tally, None)

GroupedDataFrame.summarize = _wrap(dplyr.summarize_, DataFrame)
GroupedDataFrame.summarise = GroupedDataFrame.summarize
GroupedDataFrame.ungroup = _wrap(dplyr.ungroup, DataFrame)

arrange = _make_pipe(dplyr.arrange_, DataFrame)
count = _make_pipe(dplyr.count_, DataFrame)
distinct = _make_pipe(dplyr.distinct_, DataFrame)
mutate = _make_pipe(dplyr.mutate_, DataFrame)
transmute = _make_pipe(dplyr.transmute_, DataFrame)
filter = _make_pipe(dplyr.filter_, DataFrame)
select = _make_pipe(dplyr.select_, DataFrame)
group_by = _make_pipe(dplyr.group_by_, DataFrame)
summarize = _make_pipe(dplyr.summarize_, DataFrame)
summarise = summarize
distinct = _make_pipe(dplyr.distinct_, DataFrame)
inner_join = _make_pipe2(dplyr.inner_join, DataFrame)
left_join = _make_pipe2(dplyr.left_join, DataFrame)
right_join = _make_pipe2(dplyr.right_join, DataFrame)
full_join = _make_pipe2(dplyr.full_join, DataFrame)
semi_join = _make_pipe2(dplyr.semi_join, DataFrame)
anti_join = _make_pipe2(dplyr.anti_join, DataFrame)
union = _make_pipe2(dplyr.union, DataFrame)
intersect = _make_pipe2(dplyr.intersect, DataFrame)
setdiff = _make_pipe2(dplyr.setdiff, DataFrame)
sample_n = _make_pipe(dplyr.sample_n, DataFrame)
sample_frac = _make_pipe(dplyr.sample_frac, DataFrame)
slice = _make_pipe(dplyr.slice_, DataFrame)
tally = _make_pipe(dplyr.tally, DataFrame)

# family of functions 'src'
class DataSource(robjects.ListVector):
    """ Source of data tables (e.g., a schema in a relational database). """
    
    @property
    def tablenames(self):
        """ Call the R function dplyr::src_tbls() and return a vector
        of table names."""
        return tuple(dplyr.src_tbls(self))
    def get_table(self, name):
        """ "Get" table from a source (R dplyr's function `tbl`). """
        return DataFrame(tbl(self, name))
    
src = dplyr.src
src_desc = dplyr.src_desc
src_tbls = dplyr.src_tbls

src_local = dplyr.src_local
src_df = dplyr.src_df

def src_sql(*args, **kwargs):
    res = dplyr.src_sql(*args, **kwargs)
    return DataSource(res)

def src_mysql(*args, **kwargs):
    res = dplyr.src_mysql(*args, **kwargs)
    return DataSource(res)

def src_postgres(*args, **kwargs):
    res = dplyr.src_postgres(*args, **kwargs)
    return DataSource(res)

def src_sqlite(*args, **kwargs):
    res = dplyr.src_sqlite(*args, **kwargs)
    return DataSource(res)

# family of functions 'tbl'

tbl = dplyr.tbl
